@@ -2,13 +2,18 @@
 # Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
 # Distributed under the terms of "New BSD License", see the LICENSE file.
 
-from ctypes import c_double, c_int
-from mpi4py import MPI
-import numpy as np
-import pickle
 import sys
-import zmq
+from ctypes import c_double, c_int
+
+import numpy as np
+from executorlib.shared import (
+    interface_connect,
+    interface_receive,
+    interface_send,
+    interface_shutdown,
+)
 from lammps import lammps
+from mpi4py import MPI
 
 __author__ = "Sarath Menon, Jan Janssen"
 __copyright__ = (
@@ -42,14 +47,8 @@ atom_properties = {
     # taken directly from atom.cpp -> extract()
 }
 
-# Lammps executable
-args = ["-screen", "none"]
-if len(sys.argv) > 3:
-    args.extend(sys.argv[3:])
-job = lammps(cmdargs=args)
 
-
-def extract_compute(funct_args):
+def extract_compute(job, funct_args):
     def convert_data(val, type, length, width):
         data = []
         if type == 2:
@@ -88,47 +87,47 @@ def extract_compute(funct_args):
         raise ValueError("Local style is currently not supported")
 
 
-def get_version(funct_args):
+def get_version(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.version()
 
 
-def get_file(funct_args):
+def get_file(job, funct_args):
     job.file(*funct_args)
     return 1
 
 
-def commands_list(funct_args):
+def commands_list(job, funct_args):
     job.commands_list(*funct_args)
     return 1
 
 
-def commands_string(funct_args):
+def commands_string(job, funct_args):
     job.commands_string(*funct_args)
     return 1
 
 
-def extract_setting(funct_args):
+def extract_setting(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.extract_setting(*funct_args)
 
 
-def extract_global(funct_args):
+def extract_global(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.extract_global(*funct_args)
 
 
-def extract_box(funct_args):
+def extract_box(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.extract_box(*funct_args)
 
 
-def extract_atom(funct_args):
+def extract_atom(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         # extract atoms return an internal data type
         # this has to be reformatted
         name = str(funct_args[0])
-        if not name in atom_properties.keys():
+        if name not in atom_properties.keys():
             return []
 
         # this block prevents error when trying to access values
@@ -153,12 +152,12 @@ def extract_atom(funct_args):
         return np.array(data)
 
 
-def extract_fix(funct_args):
+def extract_fix(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.extract_fix(*funct_args)
 
 
-def extract_variable(funct_args):
+def extract_variable(job, funct_args):
     # in the args - if the third one,
     # which is the type is 1 - a lammps array is returned
     if funct_args[2] == 1:
@@ -177,21 +176,21 @@ def extract_variable(funct_args):
             return data
 
 
-def get_natoms(funct_args):
+def get_natoms(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.get_natoms()
 
 
-def set_variable(funct_args):
+def set_variable(job, funct_args):
     return job.set_variable(*funct_args)
 
 
-def reset_box(funct_args):
+def reset_box(job, funct_args):
     job.reset_box(*funct_args)
     return 1
 
 
-def gather_atoms(funct_args):
+def gather_atoms(job, funct_args):
     # extract atoms return an internal data type
     # this has to be reformatted
     name = str(funct_args[0])
@@ -217,11 +216,11 @@ def gather_atoms(funct_args):
     return np.array(data)
 
 
-def gather_atoms_concat(funct_args):
+def gather_atoms_concat(job, funct_args):
     # extract atoms return an internal data type
     # this has to be reformatted
     name = str(funct_args[0])
-    if not name in atom_properties.keys():
+    if name not in atom_properties.keys():
         return []
 
     # this block prevents error when trying to access values
@@ -243,7 +242,7 @@ def gather_atoms_concat(funct_args):
     return np.array(data)
 
 
-def gather_atoms_subset(funct_args):
+def gather_atoms_subset(job, funct_args):
     # convert to ctypes
     name = str(funct_args[0])
     lenids = int(funct_args[1])
@@ -254,7 +253,7 @@ def gather_atoms_subset(funct_args):
     for i in range(lenids):
         cids[i] = ids[i]
 
-    if not name in atom_properties.keys():
+    if name not in atom_properties.keys():
         return []
 
     # this block prevents error when trying to access values
@@ -280,95 +279,76 @@ def gather_atoms_subset(funct_args):
     return np.array(data)
 
 
-def create_atoms(funct_args):
-    # we have to process the input items
-    # args are natoms, ids, type, x, v, image, shrinkexceed
-    natoms = funct_args[0]
-    ids = funct_args[1]
-    type = funct_args[2]
-    x = funct_args[3]
-    v = funct_args[4]
-    image = funct_args[5]
-    shrinkexceed = funct_args[6]
-
-    id_lmp = (c_int * natoms)()
-    id_lmp[:] = ids
-
-    type_lmp = (c_int * natoms)()
-    type_lmp[:] = type
-
-    image_lmp = (c_int * natoms)()
-    image_lmp[:] = image
-
-    args = [natoms, id_lmp, type_lmp, x, v, image_lmp, shrinkexceed]
-    job.create_atoms(*args)
+def create_atoms(job, funct_args):
+    job.create_atoms(*funct_args)
     return 1
 
 
-def has_exceptions(funct_args):
+def has_exceptions(job, funct_args):
     return job.has_exceptions
 
 
-def has_gzip_support(funct_args):
+def has_gzip_support(job, funct_args):
     return job.has_gzip_support
 
 
-def has_png_support(funct_args):
+def has_png_support(job, funct_args):
     return job.has_png_support
 
 
-def has_jpeg_support(funct_args):
+def has_jpeg_support(job, funct_args):
     return job.has_jpeg_support
 
 
-def has_ffmpeg_support(funct_args):
+def has_ffmpeg_support(job, funct_args):
     return job.has_ffmpeg_support
 
 
-def installed_packages(funct_args):
+def installed_packages(job, funct_args):
     return job.installed_packages
 
 
-def set_fix_external_callback(funct_args):
+def set_fix_external_callback(job, funct_args):
     job.set_fix_external_callback(*funct_args)
+    return 1
 
 
-def get_neighlist(funct_args):
+def get_neighlist(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.get_neighlist(*funct_args)
 
 
-def find_pair_neighlist(funct_args):
+def find_pair_neighlist(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.find_pair_neighlist(*funct_args)
 
 
-def find_fix_neighlist(funct_args):
+def find_fix_neighlist(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.find_fix_neighlist(*funct_args)
 
 
-def find_compute_neighlist(funct_args):
+def find_compute_neighlist(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.find_compute_neighlist(*funct_args)
 
 
-def get_neighlist_size(funct_args):
+def get_neighlist_size(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.get_neighlist_size(*funct_args)
 
 
-def get_neighlist_element_neighbors(funct_args):
+def get_neighlist_element_neighbors(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return job.get_neighlist_element_neighbors(*funct_args)
 
 
-def get_thermo(funct_args):
+def get_thermo(job, funct_args):
     if MPI.COMM_WORLD.rank == 0:
         return np.array(job.get_thermo(*funct_args))
 
 
-def scatter_atoms(funct_args):
+def scatter_atoms(job, funct_args):
     name = str(funct_args[0])
     py_vector = funct_args[1]
     # now see if its an integer or double type- but before flatten
@@ -385,7 +365,7 @@ def scatter_atoms(funct_args):
     return 1
 
 
-def scatter_atoms_subset(funct_args):
+def scatter_atoms_subset(job, funct_args):
     name = str(funct_args[0])
     lenids = int(funct_args[2])
     ids = funct_args[3]
@@ -415,7 +395,7 @@ def scatter_atoms_subset(funct_args):
     return 1
 
 
-def command(funct_args):
+def command(job, funct_args):
     job.command(funct_args)
     return 1
 
@@ -483,26 +463,42 @@ def _gather_data_from_all_processors(data):
         return data
 
 
-if __name__ == "__main__":
+def _run_lammps_mpi(argument_lst):
+    index_selected = argument_lst.index("--zmqport")
+    port_selected = argument_lst[index_selected + 1]
+    if "--host" in argument_lst:
+        index_selected = argument_lst.index("--host")
+        host = argument_lst[index_selected + 1]
+    else:
+        host = "localhost"
+    argument_red_lst = argument_lst[:index_selected]
     if MPI.COMM_WORLD.rank == 0:
-        context = zmq.Context()
-        socket = context.socket(zmq.PAIR)
-        argument_lst = sys.argv
-        port_selected = argument_lst[argument_lst.index("--zmqport") + 1]
-        socket.connect("tcp://localhost:" + port_selected)
+        context, socket = interface_connect(host=host, port=port_selected)
+    else:
+        context, socket = None, None
+    # Lammps executable
+    args = ["-screen", "none"]
+    if len(argument_red_lst) > 1:
+        args.extend(argument_red_lst[1:])
+    job = lammps(cmdargs=args)
     while True:
         if MPI.COMM_WORLD.rank == 0:
-            input_dict = pickle.loads(socket.recv())
-            # with open('process.txt', 'a') as file:
-            #     print('Input:', input_dict, file=file)
+            input_dict = interface_receive(socket=socket)
         else:
             input_dict = None
         input_dict = MPI.COMM_WORLD.bcast(input_dict, root=0)
-        if input_dict["c"] == "close":
+        if "shutdown" in input_dict.keys() and input_dict["shutdown"]:
             job.close()
+            if MPI.COMM_WORLD.rank == 0:
+                interface_send(socket=socket, result_dict={"result": True})
+                interface_shutdown(socket=socket, context=context)
             break
-        output = select_cmd(input_dict["c"])(input_dict["d"])
+        output = select_cmd(input_dict["command"])(
+            job=job, funct_args=input_dict["args"]
+        )
         if MPI.COMM_WORLD.rank == 0 and output is not None:
-            # with open('process.txt', 'a') as file:
-            #     print('Output:', output, file=file)
-            socket.send(pickle.dumps(output))
+            interface_send(socket=socket, result_dict={"result": output})
+
+
+if __name__ == "__main__":
+    _run_lammps_mpi(argument_lst=sys.argv)
